//
// FeedParser
//
// This class does the heavy lifting of separating the Companion feed into
// component elements.
// The parse method is invoked with the raw data from the Yahoo server.
// It will create an Array of objects that represent the data from the server.
// It will invoke the display() method on the displayer object to render those objects.
//
// $Id: class_FeedParser.js,v 1.10 2004/10/28 02:41:38 dviner Exp $

function FeedParser (displayer)
{
    this.data = "";
    this.lines = new Array();
    this.displayer = displayer;
    this.charset = "ISO-8859-1";
    
    // this will show lots of parsing as it happens
    // just set it to any non-null value
    this.debug = null;
}


FeedParser.prototype.save = function(txt)
{
    var aFile = Components.classes["@mozilla.org/file/local;1"].createInstance();
    if (!aFile) return false;
    var aLocalFile=aFile.QueryInterface(Components.interfaces.nsILocalFile);
    if (!aLocalFile) return false;
    
    // where to save the data ?
    var dirServ = Components.classes['@mozilla.org/file/directory_service;1'].createInstance();
    dirServ = dirServ.QueryInterface(Components.interfaces.nsIProperties);
    

    // see http://lxr.mozilla.org/seamonkey/source/xpcom/io/nsAppDirectoryServiceDefs.h
    // and http://lxr.mozilla.org/seamonkey/source/xpcom/io/nsDirectoryServiceDefs.h
    // for additional Directory Service names.
    // ProfD is the profile directory.
    // On windows this might be
    // C:\Documents and Settings\USER\Application Data\Mozilla\Profiles\default\lzoq0znl.slt
    // On unix it might be:
    // $HOME/.mozilla/default/lzoq0znl.slt
    var processDir = dirServ.get("ProfD", Components.interfaces.nsIFile);
    
    processDir.append("yahootoolbar_saves");
    if(! processDir.exists())
        processDir.create(processDir.DIRECTORY_TYPE, 0766);
    
    // now we init the LocalFile with the dir we just made
    processDir.append("raw_companion_feed.txt");
    aLocalFile.initWithFile(processDir);

    // make the file if its not there.
    if(!aLocalFile.exists())
        aLocalFile.create(aLocalFile.NORMAL_FILE_TYPE, 0766);
        
    // now we write out the text.
    var stream = Components.classes['@mozilla.org/network/file-output-stream;1'].createInstance(Components.interfaces.nsIFileOutputStream);
    stream.init(aLocalFile, 2, 0x200, false); // open as "write only"
    stream.write(txt, txt.length);
    stream.close();
    
}

// returns the contents of the saved yahoo feed
FeedParser.prototype.readSaveFile = function()
{
    var dirServ = Components.classes['@mozilla.org/file/directory_service;1'].createInstance();
    dirServ = dirServ.QueryInterface(Components.interfaces.nsIProperties);
    
    var theFile = dirServ.get("ProfD", Components.interfaces.nsIFile);    
    theFile.append("yahootoolbar_saves");

    if (!theFile.exists())
        theFile.create(theFile.DIRECTORY_TYPE, 0766);
    
    // now we init the LocalFile with the dir we just made
    theFile.append("raw_companion_feed.txt");

    var inputStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();
    inputStream = inputStream.QueryInterface(Components.interfaces.nsIFileInputStream);
 
    inputStream.init(theFile, 1, 0, false);    // open read only
   
    var scriptableInputStream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance();
    scriptableInputStream = scriptableInputStream.QueryInterface(Components.interfaces.nsIScriptableInputStream);
   
    scriptableInputStream.init(inputStream);    // open read only
   
    // suck in the entire file
    var fileSize = scriptableInputStream.available();
    var fileContents = scriptableInputStream.read(fileSize);
    
    return fileContents;
}

// parse is responsible for parsing the whole line
FeedParser.prototype.parse = function(fdata)
{
    var ToolBarObjects = new Array();
    var offline = false;    
    var uc =
      Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    
    uc.charset = this.displayer.charset;
    

    // class_StreamListener calls parse with an empty fdata parameter
    // when the browser is offline and cannot get the yahoo feed
    if (fdata == "") {
        offline = true;
        // read in the contents of the saved feed if available and
        // use that to build the toolbar when the browser is offline
        fdata = this.readSaveFile();
    }

    //alert("Parsing!");
    this.data = fdata;
    this.save(fdata);
    this.lines = fdata.split("\n");
    var toolbarLine = this.lines[3];
    // now a single item in the toolbar is from CTRL-T to CTRL-U
    // so now we just find each of those...
    var ToolBarItems = toolbarLine.split('\x15');

	// now let's load user settings
	this.loadCompanionSettings(ToolBarItems);
	
    // now we need to determine what each item is...
    //alert('we have ' + ToolBarItems.length + ' items to look at');
    var i = 0;
    var numItems = ToolBarItems.length;
	
    for( i=0; i < numItems ; i++)
    {
        
        var item = uc.ConvertToUnicode(ToolBarItems[i]);
        // all items have this general layout:
        // Y! ^X <text> ^X TYPE ...
        // so first let's fetch the text
        
        var start = item.indexOf('\x18');
        start++;
        var end = item.indexOf('\x18',start);
        var text = item.substr(start,(end-start));
        
        // now find the TYPE.  this is always the first char after
        // the ^X... 
        var type = item.charAt(end+1);
        var typeTxt;
        
        // could be a button, menu, buttonmenu, editbox
        typeTxt = this.determineType(type);
        // now if this is a button, figure out which kind
        if(typeTxt == "button")
        {
            // end points at the ^X.  so, lets find the next ^X
            start = end+3;
            var wholeButton = item.substr(start);
            var but = new Button(text,wholeButton);
            ToolBarObjects.push(but);
            if(this.debug != null)
                alert('BUTTON\n'
                      + 'Title: '+but.title+'\n'
                      + 'Button type: '+but.type+'\n'
                      + 'Link: ' + but.link + '\n'
                      + 'Image: ' + but.img + '\n'
                      + 'Whole Line (#'+i+'): ' + item);
        }
        else if(typeTxt == "menu")
        {
            start = end+3;
            var wholeMenu = item.substr(start);
            var menu = new Menu(text, wholeMenu);
            ToolBarObjects.push(menu);		
            if(this.debug != null)
                alert('MENU\n'
                      + 'Title: '+menu.title+'\n'
                      + 'Menu type: '+menu.type+'\n'
                      + 'Menu has ' + menu.items.length + ' items in it\n'
                      + menu.dumpItems() + '\n'
                      + 'Whole Line (#'+i+'): ' + item);
        }
        else if(typeTxt == "buttonmenu")
        {
            start = end+3;
            var wholeButMenu = item.substr(start);
            var bm = new ButtonMenu(text,wholeButMenu);
            ToolBarObjects.push(bm);
            if(this.debug != null)
                alert('ButtonMenu: \n'
                    + 'button title: ' + bm.title + '\n'
                    + 'button img: ' + bm.button.img + '\n'
                    + 'button link: ' + bm.button.link + '\n'
                    + 'menu has ' + bm.menu.items.length + ' items\n'
                    + 'menu contents: \n' + bm.menu.dumpItems()
                    //+ 'RAW: ' + txt
                    );
        }
        else if(typeTxt == "editbox")
        {
       	    start = end+3;
       	    // We probably want to build out the EditBox class and let it 
       	    // do the parsing for width and link.  For now, I do it here.
	    end = item.indexOf('\x18',start);
            var width = item.substr(start,(end-start));
            ToolBarObjects.push(new EditBox(width));
        }
        else
        {
            switch(text)
            {
                case "notext":
                case "charset":
                case "uid":
                case "poll":
                   break;
                case "lang":
                   var lang = item.split("\x18");
                   switch (lang[4])
                   {
                      case "tw":
                      case "b5":
                      case "hk":
                         this.displayer.charset = "big5";
                         break;
                      case "cn":
   			 this.displayer.charset = "chinese";
                         break;
                      case "ru":
                         this.displayer.charset = "cyrillic";
                         break;
                      case "kr":
                         this.displayer.charset = "korean";
                         break;
                      case "gr":
                         this.displayer.charset = "greek";
                         break;
                      default:
                         this.displayer.charset = "iso-8859-1";
                   }
                   uc.charset = this.displayer.charset;
                   break;
                case "mtext":
                case "ctext":
                case "fontsize":
                case "messenger":
                    //alert("We know... we don't know what to do with this...\n" + item);
                    // Messenger is a link for something called "Get the Yahoo! Messenger Friends Bar"
                    // This is a little sidebar which is a little window w/ your Messenger list in it.
                    // If someone makes a Mozilla version of that, then we could add it to our toolbar.
                    break;
                default:
                    if(this.debug != null)
                        alert("TRULY unknown type! element of type: " + typeTxt + "\n" + item);
                    break;
            }
        }
    }

    // if we're offline, then add the offline label
    if (offline) {
        ToolBarObjects.push(new Label());
    }
    //alert("Done parsing!");
    // now we need to decode each item in the ToolBarItems array.
    this.displayer.display(ToolBarObjects);
    delete this.displayer;
    if (this.debug != null)
    	alert("Exiting FeedParser");
}

// this is a hack and it'll only work in English...
// apply what settings we can
FeedParser.prototype.loadCompanionSettings = function(feed)
{
	try {
		// Use icons and text or just icons?	
		var reSetting = /FALSE/;
		if (!reSetting.test(feed[0])) {
			this.displayer.UseIconsAndNames = false;
		} else {
			this.displayer.UseIconsAndNames = true;
		}
	} catch (e) {
	}
}

// determineType takes a character, and returns a string
// that represents what the item is supposed to be in the toolbar.
// ^B is a button
// ^C is a menu
// ^D is a button and a menu
FeedParser.prototype.determineType = function(txt)
{
    var code = txt.charCodeAt(0);
    switch(code)
    {
        case 2:
        {
            //alert("Char code: " + code + " is a button");
            return "button";
        }
        case 3:
        {
            //alert("Char code: " + code + " is a menu");
            return "menu";
        }
        case 4:
        {
            //alert("Char code: " + code + " is a buttonmenu");
            return "buttonmenu";
        }
        case 7:
        {
            //alert("Char code: " + code + " is a editbox");
            return "editbox";
        }
        case 131:
        {
            //alert("HEY THIS IS MESSENGER! (131)");
            return "messenger";
        }
        default:
        {
            //alert("Char code: " + code + " is unknown");
            return "unknown";
        }
    }
}
